# Nully

<img src="http://upload.wikimedia.org/wikipedia/commons/0/02/Panneau_Nully.JPG" width="200" />

[![Code Climate](https://codeclimate.com/github/tombroomfield/Nully/badges/gpa.svg)](https://codeclimate.com/github/tombroomfield/Nully)
[![Gem Version](https://badge.fury.io/rb/nully.svg)](http://badge.fury.io/rb/nully)

The beautiful fusion of the Null Object and Delegation programming patterns.

Sick of your methods returning ```nil``` and messing up your methof chains?

```ruby
User.first.business.name
No method error, undefined method for nil:NillClass
```

Clean them up with Nully and protect yourself from nils.

This gem functions in a similar way to the #delegate methods in rails, except that the delegation will only occur if that current objects implementation of that method would return nil.

```ruby
class User
  nully :business, to: Business.new
end

class business
  nully :name, to: proc { Business.last }
end
```
These methods will only get called if the user has no business and the business has no name.


## Installation

Add this line to your application's Gemfile:

    gem 'nully'

And then execute:

    $ bundle install

Or install it yourself as:

    $ gem install nully

## Usage

``` ruby
class User
  nully :name, to: NullUser.new
end

class NullUser
  def name
    'This user has no name.'
  end
end
```

Now, instead of this:
``` html+ruby
<% if @user.name %>
  <%= @user.name %>
<% else %>
 'This user has no name'
<% end %>
```
We get this:
``` html+ruby
<%= @user.name %>
```
Nice.

## Options

Currently, Nully accepts the :delegate_on_false option, defaulting to false. This will delegate if the response of the original method if nil or false.

``` ruby
class User
  nully :admin?, to: NullUser.new, delegate_on_false: true
end
```

You can also pass a symbol which will call a method on the local class. Remember
that this will expect an object to be returned that it can call the 'name' method on
to be executed on delegation.

``` ruby
class User
  nully :name, to: :parent_user
end
```


This can be expanded to allow chained NullObjects, where a NullUser could have a company method, returning a new NullCompany, which could in turn, have a set of default methods.

Confident Ruby!

## Contributing

1. Fork it ( https://github.com/tombroomfield/Nully/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request

## Upcoming features

1. Give the option to delegate on any condition.
