require 'spec_helper'
require_relative '../lib/dummy/dummy_string'
require_relative '../lib/dummy/dummy_catcher'

describe Nully do
  describe '#nully' do
    context 'When a class has the nully implementation' do
      context 'When it should delegate' do
        it 'delegates when it returns nil' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.downcase!).to eq 'Caught from dummy'
        end

        it 'delegates with attributes' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.chomp!('foo')).to eq 'Can\'t chomp foo!'
        end
      end

      context 'When it should not delegate' do
        it 'It should not be caught' do
          dumb_string = DummyString.new('HELLO')
          expect(dumb_string.downcase!).to eq 'hello'
        end

        it 'don\t delegate with attributes' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.chomp('llo')).to eq 'he'
        end

        it 'should not delegate when returning false' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.end_with?('hi')).to eq false
        end
      end

      context 'When delegate_on_false is true' do
        it 'should delegate on a false method' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.empty?).to eq 'Call on false'
        end
      end

      context 'When delegating to procs' do
        it 'Should execute the proc only when required' do
          dumb_string = DummyString.new('9999')
          expect(dumb_string.swapcase!).to eq '_NULLY_'
        end
      end

      context 'When ignoring the call on the second delegation' do
        it 'works for object'  do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.lstrip!).to eq 'unmodded string'
        end

        it 'works for methods' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.rstrip!).to eq 'nothing changed'
        end

        it 'works for procs' do
          dumb_string = DummyString.new('hello')
          expect(dumb_string.gsub!('_','!').call).to eq 'No Changes'
        end
      end

      context 'When delegating methods' do
        it 'should be able to delegate to a local method' do
          dumb_string = DummyString.new('Hello')
          expect(dumb_string.capitalize!).to eq 'Catching cap'
        end

        it 'should work when delegating with args' do
          dumb_string = DummyString.new('Hello')
          expect(dumb_string.delete!('num')).to eq 'Cant delete num from Hello'
        end
      end
    end
  end
end
